@@ -11,7 +11,7 @@
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '4.23'; # also change in Devel::NYTProf::Core
+our $VERSION = '5.00'; # also change in Devel::NYTProf::Core
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -204,8 +204,8 @@ For each subroutine called, separate counts and durations are stored I<for each
 location that called the subroutine>.
 
 Subroutine entry is detected by intercepting the C<entersub> opcode. Subroutine
-exit is detected via perl's internal save stack. The result is both extremely
-fast and very robust.
+exit is detected via perl's internal save stack. As a result the subroutine
+profiler is both fast and robust.
 
 =head3 Subroutine Recursion
 
@@ -370,13 +370,12 @@ If you find any other examples of the effect of optimizer on NYTProf output
 
 Set to 0 to disable the collection of subroutine caller and timing details.
 
-=head2 blocks=0
+=head2 blocks=1
 
-Set to 0 to disable the determination of block and subroutine location per statement.
-This makes the profiler about 50% faster (as of July 2008) and produces smaller
-output files, but you lose some valuable information. The extra cost is likely
-to be reduced in later versions anyway, as little optimization has been done on
-that part of the code.
+Set to 1 to enable the determination of block and subroutine location per statement.
+This makes the profiler about 50% slower (as of July 2008) and produces larger
+output files, but you gain some valuable insight in where time is spent in the
+blocks within large subroutines and scripts.
 
 =head2 stmts=0
 
@@ -387,6 +386,25 @@ This significantly reduces the overhead of the profiler and can also be useful
 for profiling large applications that would normally generate a very large
 profile data file.
 
+=head2 calls=N
+
+This option is I<new and experimental>.
+
+With calls=1 (the default) subroutine call I<return> events are emitted into
+the data stream as they happen.  With calls=2 subroutine call I<entry> events
+are also emitted. With calls=0 no subroutine call events are produced.
+This option depends on the C<subs> option being enabled, which it is by default.
+
+The L<nytprofcalls> utility can be used to process this data. It too is I<new
+and experimental> and so likely to change.
+
+The subroutine profiler normally gathers data in memory and outputs a summary
+when the profile data is being finalized, usually when the program has finished.
+The summary contains aggregate information for all the calls from one location
+to another, but the details of individual calls have been lost.
+The calls option enables the recording of individual call events and thus
+more detailed analysis and reporting of that data.
+
 =head2 leave=0
 
 Set to 0 to disable the extra work done by the statement profiler
@@ -496,9 +514,10 @@ When perl exits normally it runs any code defined in C<END> blocks.
 NYTProf defines an END block that finishes profiling and writes out the final
 profile data.
 
-If the process ends due to a signal then END blocks are not executed.
-The C<sigexit> option tells NYTProf to catch some signals (e.g. INT, HUP, PIPE,
-SEGV, BUS) and ensure a usable by executing:
+If the process ends due to a signal then END blocks are not executed so the
+profile will be incomplete and unusable.  The C<sigexit> option tells NYTProf
+to catch some signals (e.g. INT, HUP, PIPE, SEGV, BUS) and ensure a usable
+profile by executing:
 
     DB::finish_profile();
     exit 1;
@@ -767,20 +786,20 @@ You can reduce the cost of profiling by adjusting some options. The trade-off
 is reduced detail and/or accuracy in reports.
 
 If you don't need statement-level profiling then you can disable it via L</stmts=0>.
-If you do want it but don't mind loosing block-level timings then set L</blocks=0>.
 To further boost statement-level profiling performance try L</leave=0> but note that
 I<will> apportion timings for some kinds of statements less accurate).
 
+If you don't need calls stacks or flamegrph then disable it via L</calls=0>.
 If you don't need subroutine profiling then you can disable it via L</subs=0>.
 If you do need it but don't need timings for perl opcodes then set L</slowops=0>.
 
-Generally speaking, setting blocks=0 and slowops=0 will give you a useful boost
+Generally speaking, setting calls=0 and slowops=0 will give you a useful boost
 with the least loss of detail.
 
 Another approach is to only enable NYTProf in the sections of code that
 interest you. See L</RUN-TIME CONTROL OF PROFILING> for more details.
 
-To speed up L<nytprofhtml> try using the --minimal (-m) option.
+To speed up L<nytprofhtml> try using the --minimal (-m) or --no-flame options.
 
 =head1 REPORTS
 
@@ -800,15 +819,15 @@ Creates attractive, richly annotated, and fully cross-linked html
 reports (including statistics, source code and color highlighting).
 This is the main report generation tool for NYTProf.
 
-=head2 nytprofcsv
-
-Creates comma delimited profile reports. Old and limited.
-
 =head2 nytprofcg
 
 Translates a profile into a format that can be loaded into KCachegrind
 L<http://kcachegrind.sourceforge.net>
 
+=head2 nytprofcalls
+
+Reads a profile and processes the calls events it contains.
+
 =head2 nytprofmerge
 
 Reads multiple profile data files and writes out a new file containing the merged profile data.
@@ -980,6 +999,10 @@ Why 'realtime' can appear to go backwards:
 
   http://preview.tinyurl.com/5wawnn
 
+The PostgreSQL pg_test_timing utility documentation has a good summary of timing issues:
+
+  http://www.postgresql.org/docs/9.2/static/pgtesttiming.html
+
 =for comment
 http://preview.tinyurl.com/5wawnn redirects to:
 http://groups.google.com/group/comp.os.linux.development.apps/tree/browse_frm/thread/dc29071f2417f75f/ac44671fdb35f6db?rnum=1&_done=%2Fgroup%2Fcomp.os.linux.development.apps%2Fbrowse_frm%2Fthread%2Fdc29071f2417f75f%2Fc46264dba0863463%3Flnk%3Dst%26rnum%3D1%26
