@@ -7,17 +7,20 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id: Core.pm 1345 2010-09-12 12:01:13Z tim.bunce@gmail.com $
+# $Id$
 ###########################################################
 package Devel::NYTProf::Core;
 
 
 use XSLoader;
 
-our $VERSION = '4.05';    # increment with XS changes too
+our $VERSION = '5.00';    # increment with XS changes too
 
 XSLoader::load('Devel::NYTProf', $VERSION);
 
+# Fudging for https://rt.cpan.org/Ticket/Display.html?id=82256
+$Devel::NYTProf::StrEvalTestPad = ($] <= 5.017004) ? ";\n" : "";
+
 if (my $NYTPROF = $ENV{NYTPROF}) {
     for my $optval ( $NYTPROF =~ /((?:[^\\:]+|\\.)+)/g) {
         my ($opt, $val) = $optval =~ /^((?:[^\\=]+|\\.)+)=((?:[^\\=]+|\\.)+)\z/;
